@@ -1,4 +1,4 @@
-#' Probabilistic Reversal Learning Task (Glascher et al, 2009)
+#' Probabilistic Reversal Learning Task
 #' 
 #' @description 
 #' Hierarchical Bayesian Modeling of the Probabilistic Reversal Learning (PRL) Task using the following parameters: "eta" (learning rate), "alpha" (indecision point), "beta" (inverse temperature).
@@ -6,7 +6,7 @@
 #' \strong{MODEL:}
 #' Fictitious Update Model (Glascher et al., 2009, Cerebral Cortex)
 #' 
-#' @param data A .txt file containing the data to be modeled. Data columns should be labelled as follows: "subjID", "choice", and "rewlos". See \bold{Details} below for more information. 
+#' @param data A .txt file containing the data to be modeled. Data columns should be labelled as follows: "subjID", "choice", and "outcome". See \bold{Details} below for more information. 
 #' @param niter Number of iterations, including warm-up.
 #' @param nwarmup Number of iterations used for warm-up only.
 #' @param nchain Number of chains to be run.
@@ -16,15 +16,20 @@
 #' @param indPars Character value specifying how to summarize individual parameters. Current options are: "mean", "median", or "mode".
 #' @param saveDir Path to directory where .RData file of model output (\code{modelData}) can be saved. Leave blank if not interested.
 #' @param email Character value containing email address to send notification of completion. Leave blank if not interested. 
-#' 
-#' @return \code{modelData}  A class \code{'hBayesDM'} object with the following components:
+#' @param modelRegressor Exporting model-based regressors? TRUE or FALSE. Currently not available for this model.
+#' @param adapt_delta Floating point number representing the target acceptance probability of a new sample in the MCMC chain. Must be between 0 and 1. See \bold{Details} below.
+#' @param stepsize Integer value specifying the size of each leapfrog step that the MCMC sampler can take on each new iteration. See \bold{Details} below.
+#' @param max_treedepth Integer value specifying how many leapfrog steps that the MCMC sampler can take on each new iteration. See \bold{Details} below. 
+#'  
+#' @return \code{modelData}  A class \code{"hBayesDM"} object with the following components:
 #' \describe{
-#'  \item{\code{model}}{Character string with the name of the model ("prl_fictitious").}
-#'  \item{\code{allIndPars}}{\code{'data.frame'} containing the summarized parameter 
-#'    values (as specified by \code{'indPars'}) for each subject.}
-#'  \item{\code{parVals}}{A \code{'list'} where each element contains posterior samples
+#'  \item{\code{model}}{Character string with the name of the model (\code{"prl_fictitious"}).}
+#'  \item{\code{allIndPars}}{\code{"data.frame"} containing the summarized parameter 
+#'    values (as specified by \code{"indPars"}) for each subject.}
+#'  \item{\code{parVals}}{A \code{"list"} where each element contains posterior samples
 #'    over different model parameters. }
-#'  \item{\code{fit}}{A class \code{'stanfit'} object containing the fitted model.}
+#'  \item{\code{fit}}{A class \code{"stanfit"} object containing the fitted model.}
+#'  \item{\code{rawdata}}{\code{"data.frame"} containing the raw data used to fit the model, as specified by the user.}
 #' }
 #' 
 #' @importFrom rstan stan rstan_options extract
@@ -40,12 +45,12 @@
 #' (e.g. ".txt"), that contains the behavioral data of all subjects of interest for the current analysis. 
 #' The file should be a text (.txt) file whose rows represent trial-by-trial observations and columns 
 #' represent variables. For the Probabilistic Reversal Learning Task, there should be three columns of data 
-#' with the labels "subjID", "choice", and "rewlos". It is not necessary for the columns to be in this particular order, 
+#' with the labels "subjID", "choice", and "outcome". It is not necessary for the columns to be in this particular order, 
 #' however it is necessary that they be labelled correctly and contain the information below:
 #' \describe{
 #'  \item{\code{"subjID"}}{A unique identifier for each subject within data-set to be analyzed.}
 #'  \item{\code{"choice"}}{An integer value representing the chosen choice option within the given trial (e.g., 1 or 2 in PRL).}
-#'  \item{\code{"rewlos"}}{A 1 or -1 for outcome within each given trial (1 = reward, -1 = loss).}
+#'  \item{\code{"outcome"}}{A 1 or -1 for outcome within each given trial (1 = reward, -1 = loss).}
 #' } 
 #' \strong{*}Note: The data.txt file may contain other columns of data (e.g. "Reaction_Time", "trial_number", etc.), but only the data with the column
 #' names listed above will be used for analysis/modeling. As long as the columns above are present and labelled correctly,
@@ -54,20 +59,36 @@
 #' \strong{nwarmup} is a numerical value that specifies how many MCMC samples should not be stored upon the 
 #' beginning of each chain. For those familiar with Bayesian methods, this value is equivalent to a burn-in sample. 
 #' Due to the nature of MCMC sampling, initial values (where the sampling chain begins) can have a heavy influence 
-#' on the generated posterior distributions. The \strong{nwarmup} argument can be set to a high number in order to curb the 
+#' on the generated posterior distributions. The \code{nwarmup} argument can be set to a high number in order to curb the 
 #' effects that initial values have on the resulting posteriors.  
 #' 
 #' \strong{nchain} is a numerical value that specifies how many chains (i.e. independent sampling sequences) should be
 #' used to draw samples from the posterior distribution. Since the posteriors are generated from a sampling 
 #' process, it is good practice to run multiple chains to ensure that a representative posterior is attained. When
 #' sampling is completed, the multiple chains may be checked for convergence with the \code{plot(myModel, type = "trace")}
-#' command. The chains should resemble a "furry caterpillar". 
+#' command. The chains should resemble a "furry caterpillar".
 #' 
 #' \strong{nthin} is a numerical value that specifies the "skipping" behavior of the MCMC samples being chosen 
-#' to generate the posterior distributions. By default, \strong{nthin} is equal to 1, hence every sample is used to 
-#' generate the posterior.
+#' to generate the posterior distributions. By default, \code{nthin} is equal to 1, hence every sample is used to 
+#' generate the posterior. 
 #' 
-#' @export  
+#' \strong{Contol Parameters:} adapt_delta, stepsize, and max_treedepth are advanced options that give the user more control 
+#' over Stan's MCMC sampler. The Stan creators recommend that only advanced users change the default values, as alterations
+#' can profoundly change the sampler's behavior. Refer to Hoffman & Gelman (2014, Journal of Machine Learning Research) for 
+#' more information on the functioning of the sampler control parameters. One can also refer to section 58.2 of the  
+#' \href{http://mc-stan.org/documentation/}{Stan User's Manual} for a less technical description of these arguments. 
+#' 
+#' @export 
+#' 
+#' @references 
+#' Glascher, J., Hampton, A. N., & O'Doherty, J. P. (2009). Determining a Role for Ventromedial Prefrontal Cortex in Encoding 
+#' Action-Based Value Signals During Reward-Related Decision Making. Cerebral Cortex, 19(2), 483-495. http://doi.org/10.1093/cercor/bhn098
+#' 
+#' Hoffman, M. D., & Gelman, A. (2014). The No-U-turn sampler: adaptively setting path lengths in Hamiltonian Monte Carlo. The 
+#' Journal of Machine Learning Research, 15(1), 1593-1623.
+#' 
+#' @seealso 
+#' We refer users to our in-depth tutorial for an example of using hBayesDM: \url{https://rpubs.com/CCSL/hBayesDM}
 #' 
 #' @examples 
 #' \dontrun{
@@ -81,26 +102,36 @@
 #' printFit(output)
 #' }
 
-prl_fictitious <- function(data     = NULL,
-                           niter    = 3000, 
-                           nwarmup  = 1000, 
-                           nchain   = 1,
-                           ncore    = 1, 
-                           nthin    = 1,
-                           inits    = "random",  
-                           indPars  = "mean", 
-                           saveDir  = NULL,
-                           email    = NULL) {
+prl_fictitious <- function(data          = "choose",
+                           niter         = 3000, 
+                           nwarmup       = 1000, 
+                           nchain        = 1,
+                           ncore         = 1, 
+                           nthin         = 1,
+                           inits         = "random",  
+                           indPars       = "mean", 
+                           saveDir       = NULL,
+                           email         = NULL,
+                           modelRegressor= FALSE,
+                           adapt_delta   = 0.8,
+                           stepsize      = 1,
+                           max_treedepth = 10 ) {
 
   # Path to .stan model file
-  modelPath <- system.file("stan", "prl_fictitious.stan", package="hBayesDM")
-
+  if (modelRegressor) { # model regressors (for model-based neuroimaging, etc.)
+    stop("** Model-based regressors are not available for this model **\n")
+  } else {
+    modelPath <- system.file("stan", "prl_fictitious.stan", package="hBayesDM")
+  }
+  
   # To see how long computations take
   startTime <- Sys.time()    
 
   # For using example data
   if (data=="example") {
     data <- system.file("extdata", "prl_exampleData.txt", package = "hBayesDM")
+  } else if (data=="choose") {
+    data <- file.choose()
   }
   
   # Load data
@@ -117,7 +148,7 @@ prl_fictitious <- function(data     = NULL,
   # Specify the number of parameters and parameters of interest 
   numPars <- 3
   POI     <- c("mu_eta", "mu_alpha", "mu_beta", 
-               "sd_eta", "sd_alpha", "sd_beta",
+               "sigma",
                "eta", "alpha", "beta", 
                "log_lik")
   
@@ -158,7 +189,7 @@ prl_fictitious <- function(data     = NULL,
     useTrials    <- Tsubj[i]
     tmp          <- subset(rawdata, rawdata$subjID == curSubj)
     choice[i, 1:useTrials] <- tmp$choice
-    rewlos[i, 1:useTrials] <- tmp$rewlos
+    rewlos[i, 1:useTrials] <- tmp$outcome
   }
   
   dataList <- list(
@@ -183,15 +214,11 @@ prl_fictitious <- function(data     = NULL,
     }  
     genInitList <- function() {
       list(
-        mu_eta_pr   = qnorm(inits_fixed[1]),
-        mu_alpha_pr = qnorm(inits_fixed[2]),
-        mu_beta_pr  = qnorm(inits_fixed[3] / 5),
-        sd_eta      = 1,
-        sd_alpha    = 1,
-        sd_beta     = 1,
-        eta_pr      = rep(qnorm(inits_fixed[1]), numSubjs),
-        alpha_pr    = rep(qnorm(inits_fixed[2]), numSubjs),
-        beta_pr     = rep(qnorm(inits_fixed[3]/5), numSubjs)
+        mu_p     = c( qnorm(inits_fixed[1]), qnorm(inits_fixed[2]), qnorm(inits_fixed[3] / 5) ),
+        sigma    = c(1.0, 1.0, 1.0),
+        eta_pr   = rep(qnorm(inits_fixed[1]), numSubjs),
+        alpha_pr = rep(qnorm(inits_fixed[2]), numSubjs),
+        beta_pr  = rep(qnorm(inits_fixed[3]/5), numSubjs)
       )
     }
   } else {
@@ -218,23 +245,29 @@ prl_fictitious <- function(data     = NULL,
   cat("************************************\n")
   
   # Fit the Stan model
-  fit <- rstan::stan(file   = modelPath,
-                     data   = dataList, 
-                     pars   = POI,
-                     warmup = nwarmup,
-                     init   = genInitList, 
-                     iter   = niter, 
-                     chains = nchain,
-                     thin   = nthin)
+  fit <- rstan::stan(file    = modelPath,
+                     data    = dataList, 
+                     pars    = POI,
+                     warmup  = nwarmup,
+                     init    = genInitList, 
+                     iter    = niter, 
+                     chains  = nchain,
+                     thin    = nthin,
+                     control = list(adapt_delta   = adapt_delta, 
+                                    max_treedepth = max_treedepth, 
+                                    stepsize      = stepsize) )
   
+  ## Extract parameters
   parVals <- rstan::extract(fit, permuted=T)
   
   eta   <- parVals$eta
   alpha <- parVals$alpha
   beta  <- parVals$beta
   
+  # Individual parameters (e.g., individual posterior means)
   allIndPars <- array(NA, c(numSubjs, numPars))
   allIndPars <- as.data.frame(allIndPars)
+  
   for (i in 1:numSubjs) {
     if (indPars=="mean") {
       allIndPars[i, ] <- c( mean(eta[, i]), 
@@ -258,8 +291,8 @@ prl_fictitious <- function(data     = NULL,
                             "subjID")
 
   # Wrap up data into a list
-  modelData        <- list(modelName, allIndPars, parVals, fit)
-  names(modelData) <- c("model", "allIndPars", "parVals", "fit")
+  modelData        <- list(modelName, allIndPars, parVals, fit, rawdata)
+  names(modelData) <- c("model", "allIndPars", "parVals", "fit", "rawdata")
   class(modelData) <- "hBayesDM"
 
   # Total time of computations
@@ -289,5 +322,3 @@ prl_fictitious <- function(data     = NULL,
   
   return(modelData)
 }
-
-
